@@ -2,7 +2,7 @@
 # debianbts.py - Routines to deal with the debbugs web pages
 #
 #   Written by Chris Lawrence <lawrencc@debian.org>
-#   (C) 1999-2004 Chris Lawrence
+#   (C) 1999-2005 Chris Lawrence
 #
 # This program is freely distributable per the following license:
 #
@@ -22,13 +22,26 @@
 #
 # Version ##VERSION##; see changelog for revision history
 #
-# $Id: debianbts.py,v 1.8 2004/06/22 22:51:52 lawrencc Exp $
+# $Id: debianbts.py,v 1.21 2005/11/30 04:28:34 lawrencc Exp $
 
 import sgmllib, glob, os, re, reportbug, rfc822, time, urllib, checkversions
 from urlutils import open_url
 from types import StringTypes
 import sys
 
+import mailbox
+import email
+import email.Errors
+import cStringIO
+
+def msgfactory(fp):
+    try:
+        return email.message_from_file(fp)
+    except email.Errors.MessageParseError:
+        # Don't return None since that will
+        # stop the mailbox iterator
+        return ''
+    
 class Error(Exception):
     pass
 
@@ -146,8 +159,8 @@ debother = {
     'general' : 'General problems (e.g., that many manpages are mode 755)',
     'install' : 'Problems with the sarge installer.',
     'installation' : 'General installation problems not covered otherwise.',
-    'kernel' : '(Obsolete, please use "kernel-image" instead.)',
-    'kernel-image' : 'Problems with the Linux kernel, or the kernel shipped with Debian',
+    'kernel' : '(Obsolete, please use "linux-image" instead.)',
+    'linux-image' : 'Problems with the Linux kernel, or the kernel shipped with Debian',
     'listarchives' :  'Problems with the WWW mailing list archives',
     'lists.debian.org' : 'The mailing lists, debian-*@lists.debian.org.',
     'mirrors' : 'Problems with Debian archive mirrors.',
@@ -158,6 +171,7 @@ debother = {
 #slink-cd -- Slink CD 
 #spam -- Spam (reassign spam to here so we can complain about it)
     'security.debian.org' : 'Problems with the security updates server',
+    'upgrade-reports' : 'Reports of successful and unsucessful upgrades',
     'wnpp' : 'Work-Needing and Prospective Packages list',
     'www.debian.org' : 'Problems with the WWW site (including other *.debian.org sites)'
     }
@@ -166,9 +180,10 @@ progenyother = {
     'debian-general' : 'Any non-package-specific bug',
     }
 
-def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
+def handle_wnpp(package, bts, ui, fromaddr, online=True, http_proxy=None):
     desc = body = ''
     headers = []
+    pseudos = []
     query = True
     
     tag = ui.menu('What sort of request is this?  (If none of these '
@@ -179,6 +194,8 @@ def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
         "The package has been `Orphaned'. It needs a new maintainer as soon as possible.",
         'RFA' :
         "This is a `Request for Adoption'. Due to lack of time, resources, interest or something similar, the current maintainer is asking for someone else to maintain this package. He/she will maintain it in the meantime, but perhaps not in the best possible way. In short: the package needs a new maintainer.",
+        'RFH' :
+        "This is a `Request For Help'. The current maintainer wants to continue to maintain this package, but he/she needs some help to do this, because his/her time is limited or the package is quite big and needs several maintainers.",
         'ITP' :
         "This is an `Intent To Package'. Please submit a package description along with copyright and URL in such a report.",
         'RFP' :
@@ -204,8 +221,9 @@ def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
         if available and (not online or checkversions.check_available(
             package, '0', http_proxy=http_proxy)):
             if not ui.yes_no(
-                ('A package called %s already appears to exist; continue?' %
-                 package), 'Ignore this problem and continue.  If you have '
+                ('A package called %s already appears to exist (at least on '
+                 'your system); continue?' % package),
+                'Ignore this problem and continue.  If you have '
                 'already locally created a package with this name, this '
                 'warning message may have been produced in error.',
                 'Exit without filing a report.', default=0):
@@ -221,6 +239,7 @@ def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
 
         if tag == 'ITP':
             headers.append('X-Debbugs-CC: debian-devel@lists.debian.org')
+            pseudos.append('Owner: %s' % fromaddr)
             ui.ewrite('Your report will be carbon-copied to debian-devel, '
                       'per Debian policy.\n')
 
@@ -233,7 +252,7 @@ def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
 
 (Include the long description here.)
 """ % (package, desc)
-    elif tag in ('O', 'RFA'):
+    elif tag in ('O', 'RFA', 'RFH'):
         severity = 'normal'
         query = False
         if not available:
@@ -250,18 +269,26 @@ def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
                 sys.exit(1)
             desc = fulldesc = ''
         else:
-            desc = info[10] or ''
-            package = info[11] or package
-            fulldesc = info[12]
+            desc = info[11] or ''
+            package = info[12] or package
+            fulldesc = info[13]
 
         if tag == 'O' and info and info[9] in \
                ('required', 'important', 'standard'):
             severity = 'important'
 
+        if tag == 'RFH':
+            headers.append('X-Debbugs-CC: debian-devel@lists.debian.org')
+            ui.ewrite('Your request will be carbon-copied to debian-devel, '
+                      'per Debian policy.\n')
+
         if fulldesc:
             orphstr = 'intend to orphan'
             if tag == 'RFA':
                 orphstr = 'request an adopter for'
+            elif tag == 'RFH':
+                orphstr = 'request assistance with maintaining'
+                
             body = ('I %s the %s package.\n\n'
                     'The package description is:\n') % (orphstr, package)
             body = body + fulldesc + '\n'
@@ -271,55 +298,46 @@ def handle_wnpp(package, bts, ui, online=True, http_proxy=None):
     else:
         subject = '%s: %s' % (tag, package)
 
-    return (subject, severity, headers, body, query)
+    return (subject, severity, headers, pseudos, body, query)
 
 # Supported servers
 # Theoretically support for GNATS and Jitterbug could be added here.
-
 SYSTEMS = { 'debian' :
             { 'name' : 'Debian', 'email': '%s@bugs.debian.org',
               'btsroot' : 'http://www.debian.org/Bugs/',
-              'query-dpkg' : 1, 'type' : 'debbugs',
-              'otherpkgs' : debother, 'nonvirtual' : ['kernel-image'],
+              'otherpkgs' : debother,
+              'nonvirtual' : ['linux-image', 'kernel-image'],
               'specials' : { 'wnpp': handle_wnpp },
               # Dependency packages
               'deppkgs' : ('gcc', 'g++', 'cpp', 'gcj', 'gpc', 'gobjc',
                            'chill', 'gij', 'g77', 'python', 'python-base',
                            'x-window-system-core', 'x-window-system'),
-              'cgiroot' : 'http://bugs.debian.org/cgi-bin/',
-              'mirrors' : {} },
+              'cgiroot' : 'http://bugs.debian.org/cgi-bin/' },
             'kde' :
             { 'name' : 'KDE Project', 'email': '%s@bugs.kde.org',
-              'btsroot': 'http://bugs.kde.org/', 'type' : 'debbugs',
-              'query-dpkg' : 1, 'otherpkgs' : {}, 'cgiroot' : None,
-              'mirrors' : {} },
+              'btsroot': 'http://bugs.kde.org/' },
             'mandrake' :
             { 'name' : 'Linux-Mandrake', 'email': '%s@bugs.linux-mandrake.com',
-              'btsroot': None,
-              'type' : 'debbugs', 'query-dpkg' : 0, 'otherpkgs' : {},
-              'cgiroot' : None, 'mirrors' : {} },
+              'type' : 'mailto', 'query-dpkg' : False },
             'gnome' :
             { 'name' : 'GNOME Project', 'email': '%s@bugs.gnome.org',
-              'type' : 'mailto', 'mirrors' : {}, 'cgiroot' : None,
-              'query-dpkg' : 0, 'otherpkgs' : {} },
+              'type' : 'mailto', 'query-dpkg' : False },
             'ximian' :
             { 'name' : 'Ximian', 'email': '%s@bugs.ximian.com',
-              'type' : 'mailto', 'mirrors' : {}, 'cgiroot' : None,
-              'query-dpkg' : 1, 'otherpkgs' : {} },
+              'type' : 'mailto' },
             'progeny' :
             { 'name' : 'Progeny', 'email' : 'bugs@progeny.com',
-              'type' : 'gnats', 'mirrors' : {}, 'cgiroot' : None,
-              'query-dpkg' : 1, 'otherpkgs' : progenyother },
+              'type' : 'gnats', 'otherpkgs' : progenyother },
+            'ubuntu' :
+            { 'name' : 'Ubuntu', 'email' : 'ubuntu-users@lists.ubuntu.com',
+              'type' : 'mailto' },
             'guug' :
             { 'name' : 'GUUG (German Unix User Group)',
-              'email' : '%s@bugs.guug.de',
-              'type' : 'debbugs', 'mirrors' : {}, 'cgiroot' : None,
-              'query-dpkg' : 0, 'otherpkgs' : {} },
-            'ubuntu' :
-            { 'name' : 'Ubuntu',
-              'email' : 'ubuntu-users@lists.ubuntu.com',
-              'type' : 'mailto', 'mirrors' : {}, 'cgiroot' : None,
-              'query-dpkg' : 1, 'otherpkgs' : {} },
+              'email' : '%s@bugs.guug.de', 'query-dpkg' : False },
+            'grml' :
+            { 'name' : 'grml', 'email': '%s@bugs.grml.org',
+              'btsroot' : 'http://bugs.grml.org/',
+              'cgiroot' : 'http://bugs.grml.org/cgi-bin/' },
             }
 
 SYSTEMS['helixcode'] = SYSTEMS['ximian']
@@ -338,9 +356,12 @@ CLASSES = {
 
 CLASSLIST = ['sw-bug', 'doc-bug', 'change-request']
 
+CRITICAL_TAGS = {
+    'security' : 'This problem is a security vulnerability in Debian.',
+}
+
 TAGS = {
     'patch' : 'You are including a patch to fix this problem.',
-    'security' : 'This problem raises a security issue.',
 ##    'upstream' : 'You believe this problem is not specific to Debian.',
 ##    'potato' : 'This bug only applies to the potato release (Debian 2.2).',
 ##    'woody' : 'This bug only applies to the woody release (Debian 3.0).',
@@ -352,9 +373,10 @@ TAGS = {
 ##    'done' : 'No more tags.',
     }
 
-EXTRA_TAGS = ['potato', 'woody', 'sarge', 'sid', 'upstream']
+EXTRA_TAGS = ['potato', 'woody', 'sarge', 'security', 'sid', 'upstream']
 
-TAGLIST = ['l10n', 'patch', 'security', 'experimental']
+TAGLIST = ['l10n', 'patch', 'experimental']
+CRITICAL_TAGLIST = ['security']
 
 def yn_bool(setting):
     if setting:
@@ -367,7 +389,7 @@ def yn_bool(setting):
 def cgi_report_url(system, number, archived=False, mbox=False):
     root = SYSTEMS[system].get('cgiroot')
     if root:
-        return '%sbugreport.cgi?bug=%d&archive=%s&mbox=%s' % (
+        return '%sbugreport.cgi?bug=%d&archived=%s&mbox=%s' % (
             root, number, archived, yn_bool(mbox))
     return None
 
@@ -384,8 +406,7 @@ def cgi_package_url(system, package, archived=False, source=False,
     repeat = yn_bool(repeatmerged)
     archive = yn_bool(archived)
 
-    return '%spkgreport.cgi?%s=%s&archive=%s&repeatmerged=%s' % (
-        root, qtype, package, archive, repeat)
+    return '%spkgreport.cgi?%s=%s&archived=%s&repeatmerged=%s&show_list_header=no&show_list_footer=no' % (root, qtype, package, archive, repeat)
 
 def package_url(system, package, mirrors=None, source=False,
                 repeatmerged=True):
@@ -423,7 +444,7 @@ for origin in glob.glob('/etc/dpkg/origins/*'):
         fp = file(origin)
         system = os.path.basename(origin)
         SYSTEMS[system] = SYSTEMS.get(system, { 'otherpkgs' : {},
-                                                'query-dpkg' : 1,
+                                                'query-dpkg' : True,
                                                 'mirrors' : {},
                                                 'cgiroot' : None } )
         for line in fp:
@@ -484,7 +505,7 @@ class BTSParser(sgmllib.SGMLParser):
 
     def handle_data(self, data):
         if self.savedata is not None:
-            self.savedata = self.savedata + data
+            self.savedata += data
 
     # --- Hooks to save data; shouldn't need to be overridden
 
@@ -500,9 +521,9 @@ class BTSParser(sgmllib.SGMLParser):
     def check_li(self):
         if self.mode == 'summary':
             data = self.save_end()
-            if data:
+            if data and data[0] == '#':
                 self.lidatalist.append(data)
-                self.bugcount = self.bugcount + 1
+                self.bugcount += 1
 
             self.lidata = False
 
@@ -522,7 +543,9 @@ class BTSParser(sgmllib.SGMLParser):
 
     def end_h2(self):
         if self.mode == 'summary':
-            self.hierarchy.append( (self.save_end(), []) )
+            hiertitle = self.save_end()
+            if 'bug' in hiertitle:
+                self.hierarchy.append( (hiertitle, []) )
         self.endh2 = True # We are at the end of a title, flag <pre>
 
     def do_br(self, attrs):
@@ -599,6 +622,54 @@ def parse_html_report(number, url, http_proxy, followups=False, cgi=True):
 
     return (title, output)
 
+# XXX: Need to handle charsets properly
+def parse_mbox_report(number, url, http_proxy, followups=False):
+    page = open_url(url, http_proxy)
+    if not page:
+        return None
+
+    # Make this seekable
+    wholefile = cStringIO.StringIO(page.read())
+
+    mbox = mailbox.UnixMailbox(wholefile, msgfactory)
+    title = ''
+
+    output = []
+    for message in mbox:
+        if not message:
+            pass
+        
+        subject = message.get('Subject')
+        if not title:
+            title = subject
+
+        date = message.get('Date')
+        fromhdr = message.get('From')
+
+        body = entry = ''
+        for part in message.walk():
+            if part.get_content_type() == 'text/plain' and not body:
+                body = part.get_payload(None, True)
+
+        if fromhdr:
+            entry += 'From: %s%s' % (fromhdr, os.linesep)
+
+        if subject and subject != title:
+            entry += 'Subject: %s%s' % (subject, os.linesep)
+
+        if date:
+            entry += 'Date: %s%s' % (date, os.linesep)
+
+        if entry:
+            entry += os.linesep
+
+        entry += body.rstrip('\n') + os.linesep
+
+        output.append(entry)
+
+    title = "#%d: %s" % (number, title)
+    return (title, output)
+
 def get_cgi_reports(package, system='debian', http_proxy='', archived=False,
                     source=False):
     page = open_url(cgi_package_url(system, package, archived, source),
@@ -620,21 +691,22 @@ def get_cgi_report(number, system='debian', http_proxy='', archived=False,
                    followups=False):
     number = int(number)
 
-    url = cgi_report_url(system, number, archived='no')
-    return parse_html_report(number, url, http_proxy, followups, cgi=True)
+    url = cgi_report_url(system, number, archived='no', mbox=True)
+    return parse_mbox_report(number, url, http_proxy, followups)
+    #return parse_html_report(number, url, http_proxy, followups, cgi=True)
 
 def get_btsroot(system, mirrors=None):
     if mirrors:
-        alternates = SYSTEMS[system]['mirrors']
+        alternates = SYSTEMS[system].get('mirrors')
         for mirror in mirrors:
             if alternates.has_key(mirror):
                 return alternates[mirror]
-    return SYSTEMS[system]['btsroot']
+    return SYSTEMS[system].get('btsroot', '')
 
 def get_reports(package, system='debian', mirrors=None,
                 http_proxy='', archived=False, source=False):
     if isinstance(package, StringTypes):
-        if SYSTEMS[system]['cgiroot']:
+        if SYSTEMS[system].get('cgiroot'):
             result = get_cgi_reports(package, system, http_proxy, archived,
                                      source)
             if result: return result
@@ -674,7 +746,7 @@ def get_reports(package, system='debian', mirrors=None,
 def get_report(number, system='debian', mirrors=None,
                http_proxy='', archived=False, followups=False):
     number = int(number)
-    if SYSTEMS[system]['cgiroot']:
+    if SYSTEMS[system].get('cgiroot'):
         result = get_cgi_report(number, system, http_proxy, archived,followups)
         if result: return result
         
@@ -682,3 +754,6 @@ def get_report(number, system='debian', mirrors=None,
     if not url: return None
 
     return parse_html_report(number, url, http_proxy, followups, cgi=False)
+
+if __name__ == '__main__':
+    print get_cgi_report(183573)
