@@ -1,7 +1,7 @@
 #
 # Reportbug module - common functions for reportbug and greportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
-#   Copyright (C) 1999-2004 Chris Lawrence
+#   Copyright (C) 1999-2005 Chris Lawrence
 #
 # This program is freely distributable per the following license:
 #
@@ -21,10 +21,15 @@
 #
 # Version ##VERSION##; see changelog for revision history
 #
-# $Id: reportbug.py,v 1.13 2004/06/22 22:35:00 lawrencc Exp $
+# $Id: reportbug.py,v 1.29 2005/06/21 22:09:37 lawrencc Exp $
+
+VERSION = "reportbug ##VERSION##"
+VERSION_NUMBER = "##VERSION##"
+COPYRIGHT = VERSION + '\nCopyright (C) 1999-2004 Chris Lawrence <lawrencc@debian.org>'
 
 import time, sys, os, locale, re, pwd, commands, shlex, debianbts, rfc822
 import socket
+import pprint
 
 from string import ascii_letters, digits
 
@@ -35,7 +40,7 @@ STATUSDB = os.path.join(DPKGLIB, 'status')
 
 # Headers other than these become email headers for debbugs servers
 PSEUDOHEADERS = ('Package', 'Version', 'Severity', 'File', 'Tags',
-                 'Justification', 'Followup-For')
+                 'Justification', 'Followup-For', 'Owner')
 
 VALID_UIS = ('newt', 'text', 'gnome')
 #VALID_UIS = ('text', 'gnome')
@@ -59,30 +64,6 @@ del mode
 
 NEWBIELINE = '*** Please type your report below this line ***'
 
-def rfcdatestr(timeval = None):
-    """Cheesy implementation of an RFC 822 date."""
-    if timeval is None:
-        timeval = time.time()
-
-    try:
-        tm = time.localtime(timeval)
-        if tm[8] > 0:
-            hrs, mins = divmod(-time.altzone/60, 60)
-        else:
-            hrs, mins = divmod(-time.timezone/60, 60)
-
-        # Correct for rounding down
-        if mins and hrs < 0:
-            hrs += 1
-
-        current = locale.setlocale(locale.LC_TIME)
-        locale.setlocale(locale.LC_TIME, "C")
-        ret = time.strftime('%a, %d %b %Y %H:%M:%S', tm)
-        locale.setlocale(locale.LC_TIME, current)
-        return ret + (" %+03d%02d" % (hrs, mins))
-    except:
-        return 'Invalid date: '+`timeval`
-
 fhs_directories = ['/', '/usr', '/usr/share', '/var', '/usr/X11R6',
                    '/usr/man', '/usr/doc', '/usr/bin']
 
@@ -126,13 +107,21 @@ def glob_escape(filename):
     filename = re.sub(r'([*?\[\]])', r'\\\1', filename)
     return filename
 
+def search_pipe(searchfile):
+    arg = commands.mkarg(searchfile)
+    if os.path.exists('/usr/bin/dlocate'):
+        pipe = os.popen('COLUMNS=79 dlocate -S %s 2>/dev/null' % arg)
+    else:
+        pipe = os.popen('COLUMNS=79 dpkg --search %s 2>/dev/null' % arg)
+    return pipe
+
 def query_dpkg_for(filename):
     try:
         x = os.getcwd()
     except OSError:
         os.chdir('/')
     searchfilename = glob_escape(filename)
-    pipe = os.popen("COLUMNS=79 dpkg --search '%s' 2>/dev/null" % searchfilename)
+    pipe = search_pipe(searchfilename)
     packages = {}
 
     for line in pipe:
@@ -217,17 +206,24 @@ def get_user_id(email='', realname=''):
             # Convert & in gecos field 4 to capitalized logname: #224231
             realname = realname.replace('&', info[0].upper())
 
+    if not realname:
+        return email
+    
+    if re.match(r'[\w\s]+$', realname):
+        return '%s <%s>' % (realname, email)
+
     return rfc822.dump_address_pair( (realname, email) )
 
 statuscache = {}
-def get_package_status(package):
-    if package in statuscache:
+def get_package_status(package, avail=False):
+    if not avail and package in statuscache:
         return statuscache[package]
     
     versionre = re.compile('Version: ')
     packagere = re.compile('Package: ')
     priorityre = re.compile('Priority: ')
     dependsre = re.compile('(Pre-)?Depends: ')
+    recsre = re.compile('Recommends: ')
     conffilesre = re.compile('Conffiles: ')
     maintre = re.compile('Maintainer: ')
     statusre = re.compile('Status: ')
@@ -242,6 +238,7 @@ def get_package_status(package):
     conffiles = []
     fulldesc = []
     depends = []
+    recommends = []
     confmode = False
     state = ''
     
@@ -249,8 +246,14 @@ def get_package_status(package):
         x = os.getcwd()
     except OSError:
         os.chdir('/')
-    output = commands.getoutput("COLUMNS=79 dpkg --status '%s' 2>/dev/null" %
-                                package)
+
+    packarg = commands.mkarg(package)
+    if avail:
+        output = commands.getoutput(
+            "COLUMNS=79 dpkg --print-avail %s 2>/dev/null" % packarg)
+    else:
+        output = commands.getoutput(
+            "COLUMNS=79 dpkg --status %s 2>/dev/null" % packarg)
 
     for line in output.split(os.linesep):
         line = line.rstrip()
@@ -260,7 +263,7 @@ def get_package_status(package):
             if line[0] != '/':
                 confmode = False
             else:
-                conffiles.append(line.split())
+                conffiles = conffiles + (line.split(),)
 
         if versionre.match(line):
             (crud, pkgversion) = line.split(": ", 1)
@@ -282,6 +285,12 @@ def get_package_status(package):
             thisdepends = [[y.split()[0] for y in x.split('|')]
                            for x in (thisdepends.split(', '))]
             depends.extend(thisdepends)
+        elif recsre.match(line):
+            (crud, thisdepends) = line.split(": ", 1)
+            # Remove versioning crud
+            thisdepends = [[y.split()[0] for y in x.split('|')]
+                           for x in (thisdepends.split(', '))]
+            recommends.extend(thisdepends)
         elif conffilesre.match(line):
             confmode = True
         elif maintre.match(line):
@@ -303,17 +312,20 @@ def get_package_status(package):
     elif origin:
         if debianbts.SYSTEMS.has_key(origin):
             vendor = debianbts.SYSTEMS[origin]['name']
-            reportinfo = (debianbts.SYSTEMS[origin]['type'],
+            reportinfo = (debianbts.SYSTEMS[origin].get('type', 'debbugs'),
                           debianbts.SYSTEMS[origin]['btsroot'])
         else:
             vendor = origin.capitalize()
     else:
         vendor = ''
 
-    info = (pkgversion, pkgavail, depends, conffiles, maintainer, installed,
-            origin, vendor, reportinfo, priority, desc, src_name,
-            os.linesep.join(fulldesc), state)
-    statuscache[package] = info
+    info = (pkgversion, pkgavail, tuple(depends), tuple(recommends),
+            tuple(conffiles),
+            maintainer, installed, origin, vendor, reportinfo, priority,
+            desc, src_name, os.linesep.join(fulldesc), state)
+
+    if not avail:
+        statuscache[package] = info
     return info
 
 dbase = []
@@ -377,6 +389,9 @@ def get_source_package(package):
     return packages
 
 def get_package_info(packages):
+    if not packages:
+        return []
+    
     packinfo = get_dpkg_database()
     pkgname = r'(?:[\S]+(?:$|,\s+))'
 
@@ -438,7 +453,8 @@ def get_package_info(packages):
         notfound = [x for x in group if x not in found]
         if len(notfound) == len(group):
             if group not in found:
-                ret.append( (' | '.join(group), 'pn', '', 'Not found.', None) )
+                ret.append( (' | '.join(group), 'pn', '<none>',
+                             '(no description available)', None) )
 
     return ret
 
@@ -451,13 +467,16 @@ def packages_providing(package):
     
     return ret
 
-def get_dependency_info(package, depends):
+def get_dependency_info(package, depends, rel="depends on"):
+    if not depends:
+        return ('\n%s %s no packages.\n' % (package, rel))
+
     dependencies = []
     for dep in depends:
         for bit in dep:
             dependencies.append( (tuple(dep), bit) )
 
-    depinfo = "\nVersions of packages %s depends on:\n" % package
+    depinfo = "\nVersions of packages %s %s:\n" % (package, rel)
 
     packs = {}
     for info in get_package_info(dependencies):
@@ -493,7 +512,7 @@ def get_changed_config_files(conffiles, nocompress=False):
             confinfo[filename] = msg
             continue
 
-        filemd5 = commands.getoutput('md5sum ' + filename).split()[0]
+        filemd5 = commands.getoutput('md5sum ' + commands.mkarg(filename)).split()[0]
         if filemd5 == md5sum: continue
 
         changed.append(filename)
@@ -517,24 +536,23 @@ def get_debian_release_info():
     dists = []
     output = commands.getoutput('apt-cache policy 2>/dev/null')
     if output:
-        mre = re.compile('\s+(\d+)\s+.*$\s+release o=Debian,a=([^,]+),',
-                         re.MULTILINE)
+        mre = re.compile('\s+(\d+)\s+.*$\s+release\s.*o=(Ubuntu|Debian),a=([^,]+),', re.MULTILINE)
         found = {}
         ## XXX: When Python 2.4 rolls around, rewrite this
         for match in mre.finditer(output):
-            try:
-                pri, dist = (int(match.group(1)),
-                             DISTORDER.index(match.group(2)))
-                found[(pri, dist)] = True
-            except ValueError:
-                #warn += 'Found unknown policy: '+str(match.groups())+'\n'
-                pass
+            pword, distname = match.group(1, 3)
+            if distname in DISTORDER:
+                pri, dist = int(pword), DISTORDER.index(distname)
+            else:
+                pri, dist = int(pword), len(DISTORDER)
+
+            found[(pri, dist, distname)] = True
 
         if found:
             dists = found.keys()
             dists.sort()
             dists.reverse()
-            dists = [(x[0], DISTORDER[x[1]]) for x in dists]
+            dists = [(x[0], x[2]) for x in dists]
             debvers = dists[0][1]
 
     if os.path.exists('/etc/debian_version'):
@@ -545,7 +563,10 @@ def get_debian_release_info():
     if debvers:
         debinfo += '  APT prefers '+debvers+'\n'
     if dists:
-        debinfo += '  APT policy: ' + ', '.join([str(x) for x in dists]) + '\n'
+        # Should wrap this eventually...
+        #policystr = pprint.pformat(dists)
+        policystr = ', '.join([str(x) for x in dists])
+        debinfo += '  APT policy: %s\n' % policystr
     if warn:
         debinfo += warn
 
@@ -564,11 +585,14 @@ def get_arch():
 def generate_blank_report(package, pkgversion, severity, justification,
                           depinfo, confinfo, foundfile='', incfiles='',
                           system='debian', exinfo=0, type=None, klass='',
-                          subject='', tags='', body='', mode=MODE_EXPERT):
+                          subject='', tags='', body='', mode=MODE_EXPERT,
+                          pseudos=None):
     un = os.uname()
     utsmachine = un[4]
     debinfo = ''
-    if debianbts.SYSTEMS[system]['query-dpkg']:
+    shellpath = realpath('/bin/sh')
+
+    if debianbts.SYSTEMS[system].get('query-dpkg', True):
         debinfo += get_debian_release_info()
         debarch = get_arch()
         if debarch:
@@ -579,6 +603,9 @@ def generate_blank_report(package, pkgversion, severity, justification,
         else:
             debinfo += 'Architecture: ? (%s)\n' % utsmachine
 
+    if shellpath != '/bin/sh':
+        debinfo += 'Shell:  /bin/sh linked to %s\n' % shellpath
+
     locinfo = []
     langsetting = os.environ.get('LANG', 'C')
     allsetting = os.environ.get('LC_ALL', '')
@@ -586,9 +613,10 @@ def generate_blank_report(package, pkgversion, severity, justification,
         if setting == 'LANG':
             env = langsetting
         else:
-            env = os.environ.get(setting, langsetting)
+            env = '%s (charmap=%s)' % (os.environ.get(setting, langsetting), commands.getoutput("locale charmap"))
+
             if allsetting and env:
-                env = "%s (ignored: LC_ALL set to %s)" % (os.environ.get(setting, langsetting), allsetting)
+                env = "%s (ignored: LC_ALL set to %s)" % (env, allsetting)
             else:
                 env = allsetting or env
         locinfo.append('%s=%s' % (setting, env))
@@ -598,7 +626,11 @@ def generate_blank_report(package, pkgversion, severity, justification,
     if debianbts.SYSTEMS[system].has_key('namefmt'):
         package = debianbts.SYSTEMS[system]['namefmt'] % package
 
-    headers = ''
+    if pseudos:
+        headers = '\n'.join(pseudos)+'\n'
+    else:
+        headers = ''
+    
     if pkgversion:
         headers += 'Version: %s\n' % pkgversion
 
@@ -615,7 +647,7 @@ def generate_blank_report(package, pkgversion, severity, justification,
         if foundfile:
             headers += 'File: %s\n' % foundfile
 
-    if mode < MODE_STANDARD:
+    if mode < MODE_ADVANCED:
         body = NEWBIELINE+'\n\n'+body
     
     report = "\n"
@@ -675,7 +707,8 @@ CONFIG_ARGS = (
     'replyto', 'http_proxy', 'smtphost', 'editor', 'debconf', 'justification',
     'sign', 'nocc', 'nocompress', 'dontquery', 'noconf', 'mirrors', 'keyid',
     'headers', 'interface', 'template', 'mode', 'check_available', 'query_src',
-    'printonly', 'offline', 'check_uid', 'smtptls', 'smtpuser', 'smtppasswd')
+    'printonly', 'offline', 'check_uid', 'smtptls', 'smtpuser', 'smtppasswd',
+    'paranoid')
 
 MUA = {
     'mutt' : 'mutt -H',
@@ -767,6 +800,8 @@ def parse_config_files():
                     args['verify'] = (token == 'verify')
                 elif token in ('check-uid', 'no-check-uid'):
                     args['check_uid'] = (token == 'check-uid')
+                elif token in ('paranoid', 'no-paranoid'):
+                    args['paranoid'] = (token == 'paranoid')
                 else:
                     sys.stderr.write('Unrecognized token: %s\n' % token)
 
@@ -775,7 +810,9 @@ def parse_config_files():
     return args
 
 def parse_bug_control_file(filename):
-    submitas = submitto = reportwith = None
+    submitas = submitto = None
+    reportwith = []
+    supplemental = []
     fh = file(filename)
     for line in fh:
         line = line.strip()
@@ -789,9 +826,11 @@ def parse_bug_control_file(filename):
         elif header == 'send-to':
             submitto = data
         elif header == 'report-with':
-            reportwith = data.split(' ')
+            reportwith += data.split(' ')
+        elif header == 'package-status':
+            supplemental += data.split(' ')
 
-    return submitas, submitto, reportwith
+    return submitas, submitto, reportwith, supplemental
 
 def cleanup_msg(dmessage, headers, type):
     pseudoheaders = []
